import {
    AfterViewInit,
    Component,
    ElementRef,
    OnInit,
    ViewChild
} from '@angular/core';

import { ActivatedRoute } from '@angular/router';

import {
    MatPaginator,
    MatSort,
    MatTableDataSource
} from '@angular/material';

import {
    debounceTime,
    distinctUntilChanged,
    startWith,
    tap,
    delay
} from 'rxjs/operators';

import { merge } from 'rxjs/observable/merge';
import { fromEvent } from 'rxjs/observable/fromEvent';

import { QuotesService } from '../services/quotes.service';
import { QuotesDataSource } from '../services/quotes.datasource';


@Component({
    selector: 'quotes',
    templateUrl: './quotes.component.html',
    styleUrls: [ './quotes.component.css' ]
})
export class QuotesComponent implements OnInit, AfterViewInit {
    dataSource: QuotesDataSource;

    displayedColumns = [
        'id',
        'text',
        'game',
        'date'
    ];

    @ViewChild(MatPaginator) paginator: MatPaginator;
    @ViewChild(MatSort) sort: MatSort;
    @ViewChild('input') input: ElementRef;

    constructor(private route: ActivatedRoute,
                private quotesService: QuotesService) { }

    ngOnInit() {
        this.dataSource = new QuotesDataSource(this.quotesService);
        this.dataSource.loadQuotes('', 'id', 'desc', 0, 10);
    }

    ngAfterViewInit() {
        this.sort.sortChange.subscribe(() => this.paginator.pageIndex = 0);

        fromEvent(this.input.nativeElement, 'keyup')
            .pipe(
                debounceTime(150),
                distinctUntilChanged(),
                tap(() => {
                    this.paginator.pageIndex = 0;
                    this.loadQuotesPage();
                })
            )
            .subscribe();

        merge(this.sort.sortChange, this.paginator.page)
            .pipe(
                tap(() => this.loadQuotesPage())
            )
            .subscribe();
    }

    loadQuotesPage() {
        this.dataSource.loadQuotes(
            this.input.nativeElement.value,
            this.sort.active,
            this.sort.direction,
            this.paginator.pageIndex,
            this.paginator.pageSize);
    }
}
